/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.secure;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryException;
import org.wso2.registry.jdbc.DatabaseConstants;

import java.sql.*;

public class HSQLDBInitializer {

    private static final Log log = LogFactory.getLog(RegistryRealmFactory.class);

    public static final String UM_USERS_TABLE = "create table um_users (id varchar(255) not null, user_name varchar(255) not null, password varchar(255) not null, primary key (id), unique(user_name));";
    public static final String UM_USER_ATTRIBUTES = "create table um_user_attributes (id varchar(255) not null, attr_name varchar(255) not null, attr_value varchar(255), user_id varchar(255) not null, primary key (id));";
    public static final String UM_ROLES_TABLE = "create table um_roles (id varchar(255) not null, role_name varchar(255) not null, primary key (id), unique(role_name));";
    public static final String UM_ROLE_ATTRIBUTES_TABLE = "create table um_role_attributes (id varchar(255) not null, attr_name varchar(255) not null, attr_value varchar(255), role_id varchar(255) not null, primary key (id));";
    public static final String UM_PERMISSIONS_TABLE = "create table um_permissions (id varchar(255) not null, resource_id varchar(255) not null, action varchar(255) not null, primary key (id));";
    public static final String UM_ROLE_PERMISSIONS_TABLE = "create table um_role_permissions (id varchar(255) not null, permission_id varchar(255) not null, is_allowed smallint not null, role_id varchar(255) not null, primary key (id));";
    public static final String UM_USER_PERMISSIONS_TABLE = "create table um_user_permissions (id varchar(255) not null, is_allowed smallint not null, permission_id varchar(255) not null, user_id varchar(255) not null, primary key (id));";
    public static final String UM_USER_ROLES_TABLE = "create table um_user_roles (role_id varchar(255) not null, user_id varchar(255) not null, primary key (user_id, role_id));";

    public static final String ALTER_ROLE_ATTRIBUTES = "alter table um_role_attributes add constraint FKD869ED19F08E7776 foreign key (role_id) references um_roles ON DELETE CASCADE;";
    public static final String ALTER_ROLE_PERMISSIONS_1 = "alter table um_role_permissions add constraint FK8ADF2902918A81D6 foreign key (permission_id) references um_permissions ON DELETE CASCADE;";
    public static final String ALTER_ROLE_PERMISSIONS_2 = "alter table um_role_permissions add constraint FK8ADF2902F08E7776 foreign key (role_id) references um_roles ON DELETE CASCADE;";
    public static final String ALTER_USER_ATTRIBUTES = "alter table um_user_attributes add constraint FK4BE38AA49B0BBA16 foreign key (user_id) references um_users ON DELETE CASCADE;";
    public static final String ALTER_USER_PERMISSIONS_1 = "alter table um_user_permissions add constraint FK86993CD7918A81D6 foreign key (permission_id) references um_permissions ON DELETE CASCADE;";
    public static final String ALTER_USER_PERMISSIONS_2 = "alter table um_user_permissions add constraint FK86993CD79B0BBA16 foreign key (user_id) references um_users ON DELETE CASCADE;";
    public static final String ALTER_USER_ROLES_1 = "alter table um_user_roles add constraint FKFCA65150A3D254B4 foreign key (user_id) references um_users ON DELETE CASCADE;";
    public static final String ALTER_USER_ROLES_2 = "alter table um_user_roles add constraint FKFCA65150F08E7776 foreign key (role_id) references um_roles ON DELETE CASCADE;";

    public void createHSQLTables() throws RegistryException {

        try {
            Class.forName(DatabaseConstants.HSQL_DRIVER_NAME);
        } catch (ClassNotFoundException e) {
            String msg = "Could not load the HSQL driver.";
            log.fatal(msg, e);
            throw new RegistryException(msg);
        }

        Connection conn = null;
        try {
            conn = DriverManager.getConnection(UserManagerConstants.HSQL_DB_URL, "sa", "");

        } catch (SQLException e) {
            String msg = "Could not get the HSQL DB connection for creating tables.";
            log.fatal(msg, e);
            throw new RegistryException(msg);
        }

        try {
            DatabaseMetaData metaData = conn.getMetaData();
            ResultSet result = metaData.getTables(null, null, "um_users", null);
            if(result.next()) {
                log.info("User Manager tables were already created.");
                return;
            }
            
        } catch (SQLException e) {
            String msg = "Could not get the metadata of the User Manager database.";
            log.fatal(msg, e);
            throw new RegistryException(msg);
        }

        try {
            Statement s = conn.createStatement();

            s.executeUpdate(UM_USERS_TABLE);
            s.executeUpdate(UM_USER_ATTRIBUTES);
            s.executeUpdate(UM_ROLES_TABLE);
            s.executeUpdate(UM_ROLE_ATTRIBUTES_TABLE);
            s.executeUpdate(UM_PERMISSIONS_TABLE);
            s.executeUpdate(UM_ROLE_PERMISSIONS_TABLE);
            s.executeUpdate(UM_USER_PERMISSIONS_TABLE);
            s.executeUpdate(UM_USER_ROLES_TABLE);

            s.executeUpdate(ALTER_ROLE_ATTRIBUTES);
            s.executeUpdate(ALTER_ROLE_PERMISSIONS_1);
            s.executeUpdate(ALTER_ROLE_PERMISSIONS_2);
            s.executeUpdate(ALTER_USER_ATTRIBUTES);
            s.executeUpdate(ALTER_USER_PERMISSIONS_1);
            s.executeUpdate(ALTER_USER_PERMISSIONS_2);
            s.executeUpdate(ALTER_USER_ROLES_1);
            s.executeUpdate(ALTER_USER_ROLES_2);
            
            s.close();

        } catch (SQLException e) {
            String msg = "Could not create tables for the User Manager.";
            log.fatal(msg, e);
            throw new RegistryException(msg);

        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignore) {}
            }
        }
    }
}
